@@ -12,7 +12,7 @@ from qrogue.game.target_factory import EnemyFactory, ExplicitTargetDifficulty, T
 from qrogue.game.world import tiles
 from qrogue.game.world.map import CallbackPack, LevelMap, rooms
 from qrogue.game.world.navigation import Coordinate, Direction
-from qrogue.util import Config, HelpText, MapConfig, MyRandom, PathConfig, Logger
+from qrogue.util import Config, HelpText, MapConfig, MyRandom, PathConfig, Logger, CommonQuestions
 
 from . import parser_util
 from .generator import DungeonGenerator
@@ -25,6 +25,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
     __DEFAULT_NUM_OF_SHOP_ITEMS = 3
     __DEFAULT_NUM_OF_RIDDLE_ATTEMPTS = 7
     __ROBOT_NO_GATES = "none"
+    __SPAWN_ROOM_ID = "SR"
 
     @staticmethod
     def __normalize_reference(reference: str) -> str:
@@ -213,6 +214,12 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
             return self.visit(ctx)
         return False    # default value is "random"
 
+    def __teleport_callback(self, map_name: str, spawn_pos: Coordinate):
+        def cb(confirm: bool):
+            if confirm:
+                self.__load_map(map_name, spawn_pos)
+        CommonQuestions.GoingBack.ask(cb)
+
     ##### load from references #####
 
     def __load_reward_pool(self, reference: str) -> CollectibleFactory:
@@ -224,8 +231,8 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
             pool = [pickup.Coin(1)]
         elif ref in ['key', 'keys']:
             pool = [pickup.Key(1)]
-        elif ref in ['hp', 'health', 'heart', 'hearts', 'healthPoints']:
-            pool = [pickup.Heart(1)]
+        elif ref in ['energy']:
+            pool = [pickup.Energy()]
         else:
             self.warning(f"Imports not yet supported: {reference}")
             # todo load from somewhere else?
@@ -310,7 +317,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
                                  "and using this one as SpawnRoom.")
                 self.__spawn_pos = Coordinate(x, y)
             return room.copy(hw_dic)
-        elif self.__normalize_reference(reference) == 'sr':
+        elif reference == self.__SPAWN_ROOM_ID:
             room = rooms.SpawnRoom(self.__load_map, None, hw_dic[Direction.North], hw_dic[Direction.East],
                                    hw_dic[Direction.South], hw_dic[Direction.West])
             if self.__spawn_pos:
@@ -414,9 +421,9 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
         elif ctx.COIN_LITERAL():
             val = self.visit(ctx.integer())
             return pickup.Coin(val)
-        elif ctx.HEALTH_LITERAL():
+        elif ctx.ENERGY_LITERAL():
             val = self.visit(ctx.integer())
-            return pickup.Heart(val)
+            return pickup.Energy(val)
         elif ctx.GATE_LITERAL():
             reference = ctx.REFERENCE().getText()
             return self.__load_gate(reference)
@@ -508,7 +515,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
             pool_id, target_difficulty = self.visit(stv_pool)
             self.__stv_pools[pool_id] = target_difficulty
         self.__default_target_difficulty = self.visit(ctx.default_stv_pool())
-        self.__default_enemy_factory = EnemyFactory(self.__cbp.start_fight, self.__default_target_difficulty)
+        self.__default_enemy_factory = EnemyFactory(self.__cbp.start_fight, self.__default_target_difficulty, 1)
 
     ##### Hallway area #####
 
@@ -619,7 +626,10 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
         return self.__load_trigger(reference)
 
     def visitMessage_descriptor(self, ctx: QrogueDungeonParser.Message_descriptorContext) -> tiles.Message:
-        times = self.visit(ctx.integer())
+        if ctx.integer():
+            times = self.visit(ctx.integer())
+        else:
+            times = -1      # = always show
         reference = ctx.REFERENCE().getText()
         message = self.__load_message(reference)
         return tiles.Message(message, times)
@@ -655,7 +665,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
         pool_id = ctx.REFERENCE(0).getText()
         if pool_id in self.__stv_pools:
             difficulty = self.__stv_pools[pool_id]
-            enemy_factory = EnemyFactory(self.__cbp.start_fight, difficulty)
+            enemy_factory = EnemyFactory(self.__cbp.start_fight, difficulty, 1)
             if ctx.REFERENCE(1):
                 pool_id = ctx.REFERENCE(1).getText()
                 if pool_id in self.__reward_pools:
@@ -769,7 +779,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
             else:
                 tile_dic[tile_str] = [tile]
 
-        # this local method is needed here for not explicitely defined enemies
+        # this local method is needed here for not explicitly defined enemies
         # but since we are already in a room we don't have to reference to global data
         enemy_dic = {}
 
@@ -802,6 +812,10 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
         while len(tile_matrix) < rooms.Room.INNER_HEIGHT:
             tile_matrix.append([tiles.Floor()] * rooms.Room.INNER_WIDTH)
 
+        if room_type is rooms.AreaType.SpawnRoom:
+            tile_matrix[int(rooms.Room.INNER_HEIGHT / 2)][int(rooms.Room.INNER_WIDTH / 2)] = \
+                tiles.Teleport(self.__teleport_callback, MapConfig.back_map_string(), None)
+
         # hallways will be added later
         room = rooms.CustomRoom(room_type, tile_matrix)
         visible, foggy = visibility
@@ -887,7 +901,7 @@ class QrogueLevelGenerator(DungeonGenerator, QrogueDungeonVisitor):
                 continue
             gates.append(self.__load_gate(ref.getText()))  # todo what about pickups?
 
-        self.__robot = TestBot(num_of_qubits, gates)
+        self.__robot = TestBot(CallbackPack.instance().game_over, num_of_qubits, gates)
 
     ##### Start area #####
 
