@@ -36,7 +36,7 @@ class QrogueCUI(py_cui.PyCUI):
 
         Pausing(self.__pause_game)
         CallbackPack(self.__start_level, self.__start_fight, self.__start_boss_fight,
-                           self.__open_riddle, self.__visit_shop)
+                           self.__open_riddle, self.__visit_shop, self.__game_over)
         SaveData()  # todo load data
         MapManager(seed, self.__show_world, self.__start_level)
         Popup.update_check_achievement_function(SaveData.instance().achievement_manager.check_achievement)
@@ -60,8 +60,7 @@ class QrogueCUI(py_cui.PyCUI):
         self.__pause = PauseMenuWidgetSet(self.__controls, self.__render, Logger.instance(), self,
                                           self.__general_continue, self.switch_to_menu)
 
-        self.__spaceship = SpaceshipWidgetSet(self.__seed, self.__controls, Logger.instance(), self, self.__render,
-                                              MapManager.instance().load_map)
+        self.__spaceship = SpaceshipWidgetSet(self.__controls, Logger.instance(), self, self.__render)
         self.__workbench = WorkbenchWidgetSet(self.__controls, Logger.instance(), self,
                                               SaveData.instance().available_robots(), self.__render,
                                               self.__continue_spaceship)
@@ -69,9 +68,9 @@ class QrogueCUI(py_cui.PyCUI):
 
         self.__explore = ExploreWidgetSet(self.__controls, self.__render, Logger.instance(), self)
         self.__fight = FightWidgetSet(self.__controls, self.__render, Logger.instance(), self, self.__continue_explore,
-                                      self.__end_of_gameplay)
+                                      self.__game_over)
         self.__boss_fight = BossFightWidgetSet(self.__controls, self.__render, Logger.instance(), self,
-                                               self.__continue_explore, self.__end_of_gameplay, self.__won_tutorial)
+                                               self.__continue_explore, self.__game_over, self.__won_tutorial)
         self.__riddle = RiddleWidgetSet(self.__controls, self.__render, Logger.instance(), self, self.__continue_explore)
         self.__shop = ShopWidgetSet(self.__controls, self.__render, Logger.instance(), self, self.__continue_explore)
 
@@ -84,12 +83,11 @@ class QrogueCUI(py_cui.PyCUI):
         def open_world_view(direction: Direction, controllable: Controllable):
             return MapManager.instance().load_map(MapConfig.hub_world(), None)
 
-        def start_tutorial(direction: Direction, controllable: Controllable):
-            return MapManager.instance().load_map(MapConfig.tutorial_level(), None)
+        def start_test_level(direction: Direction, controllable: Controllable):
+            return MapManager.instance().load_map(MapConfig.test_level(), None)
 
         self.__spaceship_map = SpaceshipMap(seed, SaveData.instance().player, open_world_view, self.__use_workbench,
-                                            start_tutorial)
-
+                                            start_test_level)
 
     def _refresh_height_width(self) -> None:
         try:
@@ -502,11 +500,13 @@ class QrogueCUI(py_cui.PyCUI):
         else:
             Logger.instance().throw(ValueError(f"Tried to start a level with a non-Robot: {robot}"))
 
-    def __end_of_gameplay(self) -> None:
-        self.switch_to_menu(None)
+    def __game_over(self) -> None:
+        Popup.message("Game Over!", "You Robot was out of energy so your mission failed. You will return to the "
+                                    "Spaceship now.")
+        self.__state_machine.change_state(State.Spaceship, None)
 
     def __won_tutorial(self) -> None:
-        self.switch_to_menu(None)
+        self.switch_to_menu(None)   # todo adapt
         bell = ColorConfig.highlight_word("Bell")
         Popup.message("You won!", f"Congratulations, you defeated {bell} and successfully played the Tutorial!")
 
@@ -523,13 +523,18 @@ class QrogueCUI(py_cui.PyCUI):
         self.__state_machine.change_state(State.Pause, None)
         if not SaveData.instance().achievement_manager.check_achievement(achievements.EnteredPauseMenu):
             Popup.message("Pause", HelpText.get(HelpTextType.Pause))
-            SaveData.instance().achievement_manager.add_to_achievement(achievements.EnteredPauseMenu, 1)
+            SaveData.instance().achievement_manager.finished_tutorial(achievements.EnteredPauseMenu)
 
     def switch_to_explore(self, data) -> None:
         if data is not None:
             map = data
             self.__explore.set_data(map)
         self.apply_widget_set(self.__explore)
+        if data is not None:
+            # we cannot do this in the same if because we need to apply the widget set first otherwise the focus will
+            # be on the wrong widget after closing the popup
+            if not SaveData.instance().achievement_manager.check_achievement(achievements.FinishedTutorial):
+                Popup.scientist_says(HelpText.get(HelpTextType.FirstLevelIntroduction))
 
     def __continue_explore(self) -> None:
         self.__state_machine.change_state(State.Explore, None)
