@@ -1,9 +1,8 @@
 import enum
 import os
 import pathlib
-import shutil
 from datetime import datetime
-from typing import Callable, List
+from typing import Callable
 
 import py_cui
 
@@ -19,36 +18,48 @@ class FileTypes(enum.Enum):
 
 
 class PathConfig:
-    __BASE_PATH = os.path.join("..", "..")
+    __DEFAULT_GAME_DATA_PATH = os.path.join(os.path.dirname(__file__), "..", "data")
+    __DEFAULT_USER_DATA_PATH = os.path.join(os.path.dirname(__file__), "..", "QrogueData")
     __DATA_FOLDER = "data"
     __LOG_FOLDER = "logs"
     __KEY_LOG_FOLDER = "keylogs"
     __SCREEN_PRINTS_FOLDER = "screenprints"
     __SAVE_DATA_FOLDER = "saves"
-    __DUNGEON_FOLDER = os.path.join("data", "dungeons")
-    __TEMPLATE_ROOMS = os.path.join("data", "rooms")
-    __TEMPLATE_HALLWAYS = os.path.join("data", "hallways")
-    __TEMPLATE_STV_POOLS = os.path.join("data", "stv_pools")
-    __TEMPLATE_REWARD_POOLS = os.path.join("data", "reward_pools")
+    __DUNGEON_FOLDER = "dungeons"
+    __TEMPLATE_ROOMS = "rooms"
+    __TEMPLATE_HALLWAYS = "hallways"
+    __TEMPLATE_STV_POOLS = "stv_pools"
+    __TEMPLATE_REWARD_POOLS = "reward_pools"
     __TEMPLATE_FILE = f"templates{FileTypes.Templates}"
-
     __SAVE_FILE_NUMERATION_SEPARATOR = "_"
 
+    __Base_Path = __DEFAULT_GAME_DATA_PATH
+    __User_Data_Path = __DEFAULT_USER_DATA_PATH
+
     @staticmethod
     def __now_str() -> str:
         return datetime.now().strftime("%d%m%Y_%H%M%S")
 
     @staticmethod
-    def create_data_folder(base_path: str) -> None:
-        pathlib.Path(base_path).mkdir(parents=True, exist_ok=True)
-        data_path = os.path.join(base_path, PathConfig.__DATA_FOLDER)
-        log_path = os.path.join(base_path, PathConfig.__LOG_FOLDER)
-        key_log_path = os.path.join(base_path, PathConfig.__KEY_LOG_FOLDER)
-        screen_prints_path = os.path.join(base_path, PathConfig.__SCREEN_PRINTS_FOLDER)
-        save_data_path = os.path.join(base_path, PathConfig.__SAVE_DATA_FOLDER)
-        if not os.path.exists(data_path):
-            base_data_path = os.path.join(os.path.dirname(__file__), "..", "data")
-            shutil.copytree(base_data_path, data_path)
+    def launch_config_path() -> str:
+        return os.path.join(os.path.dirname(__file__), "..", PathConfig.__DATA_FOLDER, "qrogue_launch.config")
+
+    @staticmethod
+    def default_base_path() -> str:
+        return PathConfig.__DEFAULT_GAME_DATA_PATH
+
+    @staticmethod
+    def default_user_data_path() -> str:
+        return PathConfig.__DEFAULT_USER_DATA_PATH
+
+    @staticmethod
+    def create_folder_structure(user_data_path: str) -> None:
+        pathlib.Path(user_data_path).mkdir(parents=True, exist_ok=True)
+        log_path = os.path.join(user_data_path, PathConfig.__LOG_FOLDER)
+        key_log_path = os.path.join(user_data_path, PathConfig.__KEY_LOG_FOLDER)
+        screen_prints_path = os.path.join(user_data_path, PathConfig.__SCREEN_PRINTS_FOLDER)
+        save_data_path = os.path.join(user_data_path, PathConfig.__SAVE_DATA_FOLDER)
+
         if not os.path.exists(log_path):
             os.mkdir(log_path)
         if not os.path.exists(key_log_path):
@@ -60,24 +71,56 @@ class PathConfig:
 
     @staticmethod
     def set_base_path(base_path: str):
-        PathConfig.__BASE_PATH = base_path
+        if os.path.exists(base_path):
+            PathConfig.__Base_Path = base_path
+        else:
+            raise FileNotFoundError(f"Given base path is not valid: {base_path}")
 
     @staticmethod
-    def load_base_path(config_location: str) -> bool:
-        if config_location is None:
-            config_location = os.path.join(os.path.dirname(__file__), "../..", "installer", "qrogue.config")
+    def set_user_data_path(user_data_path: str):
+        if os.path.exists(user_data_path):
+            PathConfig.__User_Data_Path = user_data_path
+        else:
+            raise FileNotFoundError(f"Given user data path is not valid: {user_data_path}")
+
+    @staticmethod
+    def load_paths(custom_data_path: str, custom_user_data_path: str) -> bool:
         try:
-            with open(config_location) as f:
-                content = f.readlines()
-                # ignore the last character because it is \n
-                PathConfig.set_base_path(content[1][:-1])
-            return os.path.exists(PathConfig.__BASE_PATH)
+            if custom_data_path is None or custom_user_data_path is None:
+                with open(PathConfig.launch_config_path()) as f:
+                    content = f.readlines()
+
+            if custom_data_path:
+                data_path = custom_data_path
+            else:
+                data_path = content[1]
+            if custom_user_data_path:
+                user_data_path = custom_user_data_path
+            else:
+                user_data_path = content[2]
+
+            if data_path == "\n":
+                data_path = PathConfig.default_base_path()
+            elif data_path.endswith("\n"):
+                data_path = data_path[:-1]
+            if user_data_path == "\n":
+                user_data_path = PathConfig.default_user_data_path()
+            elif user_data_path.endswith("\n"):
+                user_data_path = user_data_path[:-1]
+
+            PathConfig.set_base_path(data_path)
+            PathConfig.set_user_data_path(user_data_path)
+            return os.path.exists(PathConfig.__Base_Path) and os.path.exists(PathConfig.__User_Data_Path)
         except:
             return False
 
     @staticmethod
     def base_path(file_name: str = "") -> str:
-        return os.path.join(PathConfig.__BASE_PATH, file_name)
+        return os.path.join(PathConfig.__Base_Path, file_name)
+
+    @staticmethod
+    def user_data_path(file_name: str = "") -> str:
+        return os.path.join(PathConfig.__User_Data_Path, file_name)
 
     @staticmethod
     def new_log_file(seed: int) -> str:
@@ -92,22 +135,21 @@ class PathConfig:
     @staticmethod
     def new_screen_print(text: str):
         now_str = PathConfig.__now_str()
-        file_name = os.path.join(PathConfig.__SCREEN_PRINTS_FOLDER, f"{now_str}{FileTypes.ScreenPrint.value}")
-        PathConfig.write(file_name, now_str + "\n" + text, True, True)
+        path = os.path.join(PathConfig.__SCREEN_PRINTS_FOLDER, f"{now_str}{FileTypes.ScreenPrint.value}")
+        PathConfig.write(path, now_str + "\n" + text, may_exist=True, append=True)
 
     @staticmethod
     def new_save_file(text: str):
         now_str = PathConfig.__now_str()
         num = 0
         # todo list files of folder to find out the next number and if we delete a previous save
-        file_name = os.path.join(PathConfig.__SAVE_DATA_FOLDER,
+        path = os.path.join(PathConfig.__SAVE_DATA_FOLDER,
                                  f"qrogue-save{PathConfig.__SAVE_FILE_NUMERATION_SEPARATOR}{num}{FileTypes.Save.value}")
-        PathConfig.write(file_name, now_str + "\n" + text, False, False)
+        PathConfig.write(path, now_str + "\n" + text, may_exist=False)
 
     @staticmethod
     def find_latest_save_file() -> str:
-        folder = os.path.join(PathConfig.__BASE_PATH, PathConfig.__SAVE_DATA_FOLDER)
-        files = os.listdir(folder)
+        files = os.listdir(PathConfig.user_data_path(PathConfig.__SAVE_DATA_FOLDER))
         num = 0
         for file in files:
             file_ending_index = file.rindex(FileTypes.Save.value)
@@ -116,24 +158,12 @@ class PathConfig:
                 num = cur_num
         return f"qrogue-save{PathConfig.__SAVE_FILE_NUMERATION_SEPARATOR}{num}{FileTypes.Save.value}"
 
-    @staticmethod
-    def write(file_name: str, text: str, may_exist: bool = True, append: bool = False):
-        path = PathConfig.base_path(file_name)
-        mode = "x"
-        if may_exist:
-            if os.path.exists(path):
-                mode = "w"
-                if append:
-                    mode = "a"
-        with open(path, mode) as file:
-            file.write(text)
-
     @staticmethod
     def read_keylog_buffered(file_name: str, in_keylog_folder: bool = True, buffer_size: int = 1024) -> str:
         if not file_name.endswith(FileTypes.KeyLog.value):
             file_name += FileTypes.KeyLog.value
         if in_keylog_folder:
-            path = PathConfig.base_path(os.path.join(PathConfig.__KEY_LOG_FOLDER, file_name))
+            path = PathConfig.user_data_path(os.path.join(PathConfig.__KEY_LOG_FOLDER, file_name))
         else:
             path = file_name
         if os.path.exists(path):
@@ -154,7 +184,7 @@ class PathConfig:
             path = PathConfig.base_path(os.path.join(PathConfig.__DUNGEON_FOLDER, file_name))
         else:
             path = file_name
-        return PathConfig.read(path, in_base_path=False)
+        return PathConfig.read(path, in_user_path=False)
 
     @staticmethod
     def read_level(file_name: str, in_dungeon_folder: bool = True):
@@ -165,25 +195,37 @@ class PathConfig:
             path = PathConfig.base_path(os.path.join(PathConfig.__DUNGEON_FOLDER, file_name))
         else:
             path = file_name
-        return PathConfig.read(path, in_base_path=False)
+        return PathConfig.read(path, in_user_path=False)
 
     @staticmethod
-    def read(file_name: str, in_base_path: bool = True) -> str:
-        if in_base_path:
-            path = PathConfig.base_path(file_name)
+    def read(file_name: str, in_user_path: bool) -> str:
+        if in_user_path:
+            path = PathConfig.user_data_path(file_name)
         else:
             path = file_name
-        content = ""
         if os.path.exists(path):
             with open(path, "r") as file:
                 content = file.read()
+            return content
         else:
             raise FileNotFoundError(f"File \"{file_name}\" could not be found!")
-        return content
+
+    @staticmethod
+    def write(path: str, text: str, in_user_path: bool = True, may_exist: bool = True, append: bool = False):
+        if in_user_path:
+            path = PathConfig.user_data_path(path)  # data in base_path is static so we can only write user data
+        mode = "x"
+        if may_exist:
+            if os.path.exists(path):
+                mode = "w"
+                if append:
+                    mode = "a"
+        with open(path, mode) as file:
+            file.write(text)
 
     @staticmethod
     def delete(file_name):
-        path = PathConfig.base_path(file_name)
+        path = PathConfig.user_data_path(file_name)     # data in base_path is static so we can only write user data
         if os.path.exists(path):
             os.remove(path)
 
@@ -581,7 +623,7 @@ class Config:   # todo make singleton and handle access to other configs?
         return head
 
     @staticmethod
-    def config_file() -> str:
+    def game_config_file() -> str:
         return Config.__GAME_CONFIG
 
     @staticmethod
@@ -598,36 +640,34 @@ class Config:   # todo make singleton and handle access to other configs?
         Config.__DEBUG = True
 
     @staticmethod
-    def create(custom_config: List[str] = None):
-        text = ""
-        text += Config.__GAMEPLAY_HEAD
-        text += GameplayConfig.to_file_text()
+    def setup_user_data(custom_path: str = None):
+        """
+        Sets up the folder structure and default game config in the specified path.
+        :param custom_path: where to store and load the user data (e.g. logs, save data)
+        :return:
+        """
 
-        if custom_config:
-            config_content = custom_config
+        if custom_path:
+            user_data_path = custom_path
         else:
-            file_path = os.path.join(os.path.dirname(__file__), "../..", "installer", "qrogue.config")
-            try:
-                config_content = PathConfig.read(file_path, False).splitlines()
-            except FileNotFoundError:
-                raise FileNotFoundError("Could not find the base config file qrogue.config\nPlease download the installer "
-                                        "folder again!")
-        PathConfig.create_data_folder(config_content[1])
-        path = os.path.join(config_content[1], Config.__GAME_CONFIG)
+            user_data_path = PathConfig.user_data_path()
+
+        PathConfig.create_folder_structure(user_data_path)
+        path = os.path.join(user_data_path, Config.__GAME_CONFIG)
         if not os.path.exists(path):
+            text = ""
+            text += Config.__GAMEPLAY_HEAD
+            text += GameplayConfig.to_file_text()
             with open(path, "x") as file:
                 file.write(text)
 
     @staticmethod
-    def load(config_location: str):
-        if not PathConfig.load_base_path(config_location):
-            return 1
-
-        if not os.path.exists(os.path.join(PathConfig.base_path(), Config.__GAME_CONFIG)):
-            Config.create()
+    def load():
+        if not os.path.exists(PathConfig.user_data_path(Config.__GAME_CONFIG)):
+            Config.setup_user_data()
 
         try:
-            config = PathConfig.read(Config.__GAME_CONFIG)
+            config = PathConfig.read(Config.__GAME_CONFIG, in_user_path=True)
         except FileNotFoundError:
             raise FileNotFoundError("Could not load the game's config file!")
 
